import React, { PropsWithChildren, useContext, useEffect, useRef } from 'react';
import { renderToString } from 'react-dom/server';
import { MapViewerContext } from '../MapView';

/**
 * 自定义覆盖物
 * @param props
 * @returns
 * @description 此处重点说明,需要在html中附加事件监听，需传入 on + eventKey 的形式，如 onClick
 * 同时在需要附加事件的元素中，添加 data-event-key="set" click为禁用项，会默认附加到最外层的点击
 * @warning 注意，自定义元素必须存在宽高
 * @example
 *   <CustomMap.CustomOverlay
          position={p1}
          onClick={() => {
            dispatch.push('KeyAreaDetail', {
              title: '市委市政府',
            });
          }}
          onSet={() => {
            console.log('hello world');
          }}
        >
          <div>
            <div className={styles.markerContent} data-event-key="set">
              市委市政府<span>89</span>
            </div>
            <div className={styles.markerIcon}></div>
          </div>
        </CustomMap.CustomOverlay>
 *
 *
 */
const CustomOverlay: React.FC<PropsWithChildren<CustomOverlayProps>> = (
  props,
) => {
  const { position, children, onClick, offset, ...otherEvent } = props;
  const { mapViewer } = useContext(MapViewerContext);
  const layer = useRef<any>(null);
  /**
   * 构建htmlElement，并添加事件监听
   * @returns
   */
  const setEventToHtml = () => {
    /** 创建html元素 */
    const divEl = document.createElement('div');
    divEl.innerHTML = renderToString(children);

    /** 存在onClick 添加整体点击事件 */
    if (onClick) {
      divEl.addEventListener('click', onClick);
    }

    /** 查找所有data-event-key */
    const eventKeys = divEl.querySelectorAll('[data-event-key]');

    for (let i = 0; i < eventKeys.length; i++) {
      const eventKey = eventKeys[i].getAttribute('data-event-key');

      if (eventKey) {
        const eventKeyUpper =
          eventKey?.charAt(0).toUpperCase() + eventKey?.slice(1);
        // console.log(eventKeyUpper, i);
        if (eventKeyUpper === 'Click') {
          continue;
        }
        if (
          otherEvent[`on${eventKeyUpper}`] &&
          typeof otherEvent[`on${eventKeyUpper}`] === 'function'
        ) {
          eventKeys[i].addEventListener('click', (e) => {
            otherEvent[`on${eventKeyUpper}`](e as MouseEvent);
          });
          /** 添加点击css */
          (eventKeys[i] as HTMLElement).style.cursor = 'pointer';
        }
      }
    }
    return divEl;
  };

  useEffect(() => {
    if (!mapViewer) {
      return;
    }

    const el = setEventToHtml();

    const HtmlLabelLayer = CooGL.Layer.fromHtmlLabel({
      html: el,
      position: {
        x: position[0],
        y: position[1],
        z: 10,
      },
      topOffset: offset?.top ?? 0,
      leftOffset: offset?.left ?? 0,
    });
    mapViewer?.addLayer(HtmlLabelLayer).then((res: any) => {
      layer.current = res;
    });

    /** 移除layer */
    return () => {
      mapViewer.removeLayer(layer.current);
      layer.current = null;
    };
  }, [children, mapViewer, position]);

  return null;
};

export default CustomOverlay;
