import { MinusOutlined, PlusOutlined, ReloadOutlined } from '@ant-design/icons';
import {
  useBoolean,
  useDebounceFn,
  useEventEmitter,
  usePrevious,
} from 'ahooks';
import React, {
  createContext,
  forwardRef,
  memo,
  useEffect,
  useImperativeHandle,
  useLayoutEffect,
  useRef,
  useState,
} from 'react';
import mapEnv from '../mapEnv';
import styles from './index.less';

export const MapViewerContext = createContext<any>(null);

/** 3方图层服务 */
const layersOptionsList = [
  {
    name: '蓝色底图-成都市',
    layer: 'CimBaseMap_Vector_LSB_202312',
    code: 'cfeb25a614e64f5ba258edaeca44f268',
    TileMatrixSet: 'ChinaPublicServicesCGCS2000_CimBaseMap_Vector_LSB_202312',
  },
];

/**  @name 地图实例 */
const MapView: React.ForwardRefRenderFunction<
  MapViewMethods,
  React.PropsWithChildren<MapViewProps>
> = (props, ref) => {
  const {
    zoom,
    center,
    onClick,
    children,
    showComponent = false,
    showCustomComponent = true,
    // boundingSpherePoints,
    mode = '3D',
    customInitCallback = (viewer: any) => {},
    showOriginMap = true,
  } = props;

  const mapRef = useRef<HTMLDivElement>(null);
  /** 地图容器尺寸  */
  const [mapSize, setMapSize] = useState<{ width?: number; height?: number }>({});
  /** 地图实例 */
  const mapViewer = useRef<any>(null);
  const click$ = useEventEmitter();

  const containerRef = useRef<HTMLDivElement | null>(null);

  const previous = usePrevious(mapSize);

  const { run } = useDebounceFn(
    () => {
      if (!containerRef.current) return;
      const { width, height } = containerRef.current?.getBoundingClientRect();

      if (width !== previous?.width || height !== previous?.height) {
        console.log('useDebounceFn-run setMapSize， width', width, 'height', height);
        setMapSize(prev => ({
          ...prev,
          width: width,
          height: height,
        }));
      }
    },
    {
      wait: 1000,
    },
  );

  /**
   *
   * 监听父元素的宽高变化
   * 重新设置地图容器宽高
   */
  
  useEffect(() => {
    if (!containerRef.current) return;
    run();
    window.addEventListener('resize', run);

    // V25-4 需求要动态调整地图大小，所以需要监听父元素的宽高变化；之前的window resize侦听无效。
    let resizeTimeout: NodeJS.Timeout;
    const observer = new ResizeObserver((entries) => {
      // 使用防抖来避免连续多次触发
      clearTimeout(resizeTimeout);
      resizeTimeout = setTimeout(() => {
        const entry = entries[0];
        if (!entry) return;

        setMapSize(prev => {
          if (!props.changeSize || !props.changeSize.x1 || !props.changeSize.y1 || !props.changeSize.x2 || !props.changeSize.y2) return prev;
          if (!prev.width || !prev.height) return prev;

          // 步骤1：安全访问changeSize属性
          const { x1, y1, x2, y2 } = props.changeSize || { x1: 908, y1: 430, x2: 908, y2: 572 };
          const containerWidth = entry.contentRect.width;
          const containerHeight = entry.contentRect.height;

          // 步骤2：计算目标宽高比
          const targetRatios = [
            { ratio: x1 / y1, width: x1, height: y1 },
            { ratio: x2 / y2, width: x2, height: y2 }
          ];

          // 步骤3：计算当前宽高比
          const currentRatio = containerWidth / containerHeight;
          
          // 步骤4：找到最接近的目标比例
          interface RatioInfo {
            ratio: number;
            width: number;
            height: number;
            diff: number;
          }

          const closest = targetRatios.reduce<RatioInfo>((prev, curr) => {
            const diff = Math.abs(currentRatio - curr.ratio);
            return diff < prev.diff 
              ? { ...curr, diff } 
              : prev;
          }, { 
            ratio: 0, 
            width: 0, 
            height: 0, 
            diff: Infinity 
          });

          // 步骤5：计算缩放比例
          const scale = prev.width / closest.width;
          // const targetHeight = prev.height || 0 * scale;
          const targetHeight = closest.height * scale;
          // console.log('scale', scale, 'targetHeight', targetHeight);

          // 步骤6：保持与之前状态的宽高比一致性
          const shouldUpdate = 
            prev.width !== containerWidth ||
            prev.height !== targetHeight;

          console.log('shouldUpdate', shouldUpdate, 'prev.width', prev.width, 'prev.height', prev.height, 'targetHeight', targetHeight);

          return shouldUpdate 
            ? { width: prev.width, height: targetHeight }
            // ? { width: containerWidth, height: targetHeight }
            : prev;
        });
      }, 100); // 100ms 防抖间隔
    });

    observer.observe(containerRef.current);
    // 清理函数，停止观察
    return () => {
      window.removeEventListener('resize', run);
      observer.disconnect();
      clearTimeout(resizeTimeout);
    };
  }, [containerRef.current]);

  const [isReady, { setFalse, setTrue }] = useBoolean(false);

  /**
   * @description 暴露给父组件的地图实例
   */
  useImperativeHandle(ref, () => mapViewer.current);

  const zoomFactor = showOriginMap ? 20000 : 20000;

  /** 地图初始化 */
  const initMap = () => {
    if (!mapRef.current) {
      return;
    }
    if (!center) {
      throw new Error('地图中心点是必须的');
    }
    console.log('initMap');

    mapViewer.current = null;

    CooGL.CooServer.address = mapEnv.address;
    CooGL.CooServer.defaultAccessToken = mapEnv.token;

    /** 设置高程 */
    let z = center.z ?? 200000;

    /**根据缩放等级，更新 z 值 */
    if (zoom && zoom < 20 && zoom > 0) {
      z = zoomFactor * zoom;
    }

    mapViewer.current = new CooGL.MapViewer(mapRef.current, {
      showComponent: showComponent,
    });
    const centerP = {
      duration: 0,
      destination: {
        x: center.x,
        y: center.y,
        z: z,
      },
      orientation: {
        heading: 6.283185307179586,
        pitch: -1.5684779535383755,
        roll: 0,
      },
    };
    mapViewer.current?.zoomTo(centerP);

    showOriginMap && layersOptionsList.forEach((item) => {
      /** 添加图层 */
      mapViewer.current.addLayer(
        new CooGL.Layer.fromWmtsImagery({
          url: `${mapEnv.gisUrl}${item.code}?AppKey=657676357265784832&SERVICE=WMTS&REQUEST=GetTile&VERSION=1.0.0&Layer=${item.layer}&Style=default&Format=image/png&TileMatrixSet=${item.TileMatrixSet}&TileMatrix={TileMatrix}&TileCol={TileCol}&TileRow={TileRow}`,
          layer: item.layer,
          style: 'default',
          format: 'image/png',
          tileMatrixSetID: item.TileMatrixSet,
          tilingScheme: new CooGL.GeographicTilingScheme2({}),
          maximumLevel: 18,
        }),
      );
    });

    /** 设置地图模式 */
    if (mode === '2D') {
      mapViewer.current?.scene.morphTo2D(0);
    } else {
      mapViewer.current?.scene.morphTo3D(0);
    }

    /** 地图加载完成后，在加载覆盖物 */
    mapViewer.current.readyEvent.addEventListener(() => {
      setTrue();
    });

    /** 设置最大缩放层级 */
    mapViewer.current.camera.controller.maximumZoomDistance = 250000;
    mapViewer.current.event.click(function (mapEventTarget: any) {
      if (onClick) {
        onClick(mapEventTarget);
      }
    });

    /** 响应图标点击事件 */
    mapViewer.current.event.pick(function (mapEventTarget: any) {
      if (mapEventTarget) {
        /** 说明 */
        if (mapEventTarget.layer instanceof CooGL.ImageGraphic) {
          click$.emit(mapEventTarget.layer);
        }
      }
    });

    // initCooglLoadArea();
    // setTimeout(initCooglLoadArea, 1);
    //
    if (customInitCallback) {
    // if (!showOriginMap && customInitCallback) {
      customInitCallback(mapViewer.current);
    }
    
  };

  useEffect(() => {
    /** 已重新计算容器宽高后，重新初始化地图 */
    initMap();
  }, []);

  /** 地图层级放大 */
  const zoomIn = () => {
    mapViewer.current?.camera.zoom(0.5);
  };

  /** 地图层级缩小 */
  const zoomOut = () => {
    mapViewer.current?.camera.zoom(2);
  };
  /** 回到初始位置 */
  const resetHome = () => {
    /** 设置高程 */
    let z = center.z ?? 200000;

    /**根据缩放等级，更新 z 值 */
    if (zoom && zoom < 20 && zoom > 0) {
      z = zoomFactor * zoom;
    }

    const centerP = {
      duration: 2,
      destination: {
        x: center.x,
        y: center.y,
        z: z,
      },
      orientation: {
        heading: 6.283185307179586,
        pitch: -1.5684779535383755,
        roll: 0,
      },
    };
    mapViewer.current?.flyTo(centerP);
  };

  return (
    <MapViewerContext.Provider value={{ mapViewer: mapViewer.current, click$ }}>
      <div className={styles.container} ref={containerRef}>
        <div
          ref={mapRef}
          className={styles.map}
          style={{ width: mapSize?.width, height: mapSize?.height }}
          id="customMap"
        />
        {isReady && children}
        {showCustomComponent && (
          <div className={styles.mapController}>
            <div onClick={resetHome}>
              <ReloadOutlined />
            </div>

            <div onClick={zoomIn}>
              <PlusOutlined></PlusOutlined>
            </div>
            <div onClick={zoomOut}>
              <MinusOutlined />
            </div>
          </div>
        )}
      </div>
    </MapViewerContext.Provider>
  );
};

export default memo(forwardRef(MapView));
