import mapEnv from './mapEnv';
export function calculateCenterByBoundingRectangle(
  coordinates: [number, number][],
): [number, number] {
  let minLat = Infinity,
    maxLat = -Infinity;
  let minLon = Infinity,
    maxLon = -Infinity;

  for (const [lat, lon] of coordinates) {
    if (lat < minLat) minLat = lat;
    if (lat > maxLat) maxLat = lat;
    if (lon < minLon) minLon = lon;
    if (lon > maxLon) maxLon = lon;
  }

  const centerLat = (minLat + maxLat) / 2;
  const centerLon = (minLon + maxLon) / 2;

  return [centerLat, centerLon];
}
export function calculatePolygonCentroid(
  coordinates: [number, number][],
): [number, number] {
  const n = coordinates.length;
  if (n < 3) {
    throw new Error('至少需要三个点来计算多边形质心');
  }

  let area = 0;
  let centroidX = 0;
  let centroidY = 0;

  for (let i = 0; i < n; i++) {
    const [x1, y1] = coordinates[i];
    const [x2, y2] = coordinates[(i + 1) % n]; // 下一个点，首尾相连

    const crossProduct = x1 * y2 - x2 * y1;
    area += crossProduct;
    centroidX += (x1 + x2) * crossProduct;
    centroidY += (y1 + y2) * crossProduct;
  }

  // 计算总面积
  area = area / 2;

  // 计算质心
  centroidX = centroidX / (6 * area);
  centroidY = centroidY / (6 * area);

  return [centroidX, centroidY]; // 返回 [纬度, 经度]
}
export function calculateOptimalCenter(
  coordinates: [number, number][],
): [number, number] {
  // 获取点的边界范围
  let minLat = Infinity,
    maxLat = -Infinity;
  let minLon = Infinity,
    maxLon = -Infinity;

  for (const [lat, lon] of coordinates) {
    if (lat < minLat) minLat = lat;
    if (lat > maxLat) maxLat = lat;
    if (lon < minLon) minLon = lon;
    if (lon > maxLon) maxLon = lon;
  }

  // 参数设置：划分网格
  const latStep = (maxLat - minLat) / 100; // 将纬度划分为 100 等份
  const lonStep = (maxLon - minLon) / 100; // 将经度划分为 100 等份

  let bestRect = {
    minLat: 0,
    maxLat: 0,
    minLon: 0,
    maxLon: 0,
    pointCount: 0,
  };

  // 遍历所有可能的矩形范围
  for (let i = 0; i < 100; i++) {
    for (let j = 0; j < 100; j++) {
      const rectMinLat = minLat + i * latStep;
      const rectMaxLat = rectMinLat + latStep;
      const rectMinLon = minLon + j * lonStep;
      const rectMaxLon = rectMinLon + lonStep;

      // 统计当前矩形内的点数
      let count = 0;
      for (const [lat, lon] of coordinates) {
        if (
          lat >= rectMinLat &&
          lat <= rectMaxLat &&
          lon >= rectMinLon &&
          lon <= rectMaxLon
        ) {
          count++;
        }
      }

      // 更新包含点数最多的矩形
      if (count > bestRect.pointCount) {
        bestRect = {
          minLat: rectMinLat,
          maxLat: rectMaxLat,
          minLon: rectMinLon,
          maxLon: rectMaxLon,
          pointCount: count,
        };
      }
    }
  }

  // 计算最佳矩形的中心点
  const centerLat = (bestRect.minLat + bestRect.maxLat) / 2;
  const centerLon = (bestRect.minLon + bestRect.maxLon) / 2;

  return [centerLat, centerLon];
}
/**
 * 计算给定经纬度坐标集的包围盒，返回西经、东经、南纬、北纬
 * @param coordinates 经纬度坐标数组
 * @returns {west, east, south, north}
 */
export function calculateBoundingBox(coordinates: number[][]): {
  west: number;
  east: number;
  south: number;
  north: number;
} {
  if (coordinates.length === 0) {
    throw new Error('坐标数组不能为空');
  }

  let west = coordinates[0][1];
  let east = coordinates[0][1];
  let south = coordinates[0][0];
  let north = coordinates[0][0];

  for (const [lat, lon] of coordinates) {
    if (lon < west) west = lon;
    if (lon > east) east = lon;
    if (lat < south) south = lat;
    if (lat > north) north = lat;
  }

  return { west: south, east: north, south: west, north: east };
}
/**
 * 地理信息查询相关
 */
export function request(url: string) {
  return fetch(mapEnv.gisUrl + url).then(async (result) => await result.json());
}

//逆地理编码服务
//lngLatStr的格式为104.3124,31.4321
/**
 *
 * @param {string} lngLatStr
 * @returns
 */
export function decodeCoding(lngLatStr: string) {
  return request(
    `abe32be53abc4a2c8421d68574bf8a44?AppKey=657676357265784832&lnglat=${lngLatStr}`,
  );
}

//输入提示服务
//输入提示服务提供根据用户输入的关键词查询返回建议列表。
export function searchAssociate(query: string, region = 510100) {
  return request(
    `4a1cd23054a742f4aea3b38a027f8260?AppKey=657676357265784832&query=${query}&region=${region}`,
  );
}

//正向地理编码服务
//正向地理编码服务是通过 HTTPS 协议访问远程服务的接口，提供结构化地址向经纬度转化的能力。通常用于解决各种业务地址在GIS系统中上图展示的问题，是非空间数据向空间数据转换最常用的服务。
export function encodeCoding(address: string, region = 510100) {
  return request(
    `b4d97bade6d147749ae3b4b8c86dd68b?AppKey=657676357265784832&address=${address}&city=${region}`,
  );
}

//地址标准化服务
//地址标准化是指针对用户输入的各类业务地址，对业务地址进行切词分析，补充关键地址元素的上级结构并进行地址结构化分析，输出该地址的标准地址描述方式。
export function startStandardize(address: string, region = 510100) {
  return request(
    `57946fb776fe49e19d33903cd2d2d391?AppKey=657676357265784832&address=${address}&city=${region}`,
  );
}
